use axum::{
    extract::{Path, Query, State},
    middleware,
    routing::post,
    Extension, Json, Router,
};
use serde::{Deserialize, Serialize};

use crate::{
    gpodder::{self, EpisodeActionRepository},
    server::{
        error::{AppError, AppResult},
        gpodder::{
            auth_middleware,
            format::{Format, StringWithFormat},
            models::UpdatedUrlsResponse,
        },
        Context,
    },
};

pub fn router(ctx: Context) -> Router<Context> {
    Router::new()
        .route(
            "/{username}",
            post(post_episode_actions).get(get_episode_actions),
        )
        .layer(middleware::from_fn_with_state(ctx.clone(), auth_middleware))
}

async fn post_episode_actions(
    State(ctx): State<Context>,
    Path(username): Path<StringWithFormat>,
    Extension(user): Extension<gpodder::User>,
    Json(actions): Json<Vec<gpodder::EpisodeAction>>,
) -> AppResult<Json<UpdatedUrlsResponse>> {
    if username.format != Format::Json {
        return Err(AppError::NotFound);
    }

    if *username != user.username {
        return Err(AppError::BadRequest);
    }

    Ok(
        tokio::task::spawn_blocking(move || ctx.repo.add_episode_actions(&user, actions))
            .await
            .unwrap()
            .map(|timestamp| {
                Json(UpdatedUrlsResponse {
                    timestamp,
                    update_urls: Vec::new(),
                })
            })?,
    )
}

#[derive(Deserialize, Default)]
#[serde(default)]
struct FilterQuery {
    podcast: Option<String>,
    device: Option<String>,
    since: Option<i64>,
    aggregated: bool,
}

#[derive(Serialize)]
struct EpisodeActionsResponse {
    timestamp: i64,
    actions: Vec<gpodder::EpisodeAction>,
}

async fn get_episode_actions(
    State(ctx): State<Context>,
    Path(username): Path<StringWithFormat>,
    Extension(user): Extension<gpodder::User>,
    Query(filter): Query<FilterQuery>,
) -> AppResult<Json<EpisodeActionsResponse>> {
    if username.format != Format::Json {
        return Err(AppError::NotFound);
    }

    if *username != user.username {
        return Err(AppError::BadRequest);
    }

    Ok(tokio::task::spawn_blocking(move || {
        ctx.repo.episode_actions_for_user(
            &user,
            filter.since,
            filter.podcast,
            filter.device,
            filter.aggregated,
        )
    })
    .await
    .unwrap()
    .map(|(timestamp, actions)| Json(EpisodeActionsResponse { timestamp, actions }))?)
}
